## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions

## SpyMod
## by Seby
##
## based on Action Buttons Utilities by Talchas
## and TechConquest by Bhruic

import CvUtil
from CvPythonExtensions import *
import CvGameUtils
import CvEventInterface
import Popup as PyPopup
import ActionButtons
# globals
gc = CyGlobalContext()

class SpyMod(CvGameUtils.CvGameUtils):
        # from Action Buttons Utilities
	def cannotHandleAction(self,argsList): # this is the only place I've found that you can respond to the action buttons
		pPlot = argsList[0]
		iAction = argsList[1] # iAction is data1 from the appendMultiListButton command - it is always negative so that civ4 doesn't deal with it itself
		bTestVisible = argsList[2]
		
		button = ActionButtons._getButtonFromAction(iAction)
		if(button==None):
			return False
		button.getHandler()(CyInterface().getHeadSelectedUnit())
		CyInterface().makeInterfaceDirty()
		CyInterface().setDirty(InterfaceDirtyBits.InfoPane_DIRTY_BIT, True)
		CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True)
		return True


def stealTech(unit):
    # will contain the list of techs
    lTechDiff = []
    targetTeam = gc.getTeam(unit.plot().getTeam())
    myTeam = gc.getTeam(unit.getTeam())
    myPlayer = gc.getPlayer(unit.getOwner())
    # mission's cost
    gc.getPlayer(unit.getOwner()).changeGold(-150)

    # random percentage
    iSuccess = gc.getASyncRand().get(100, 'SpyMod')
    # success condition
    if (iSuccess < 38):
        # the spy has been caught
        CyInterface().addImmediateMessage("Your spy was caught and detained for questionning !!!", "")
        unit.kill(False, 0)
    else:
        # looking for techs which can be stolen (as in TechConquest)
        for iLoopTech in range(gc.getNumTechInfos()):
            if ((targetTeam.isHasTech(iLoopTech)) and (not myTeam.isHasTech(iLoopTech)) and (myPlayer.canResearch(iLoopTech, False))):
                lTechDiff.append(iLoopTech)

        # current algo for stolen points:
        # 100 + rand(SizeCity * 70) * bonus
        # (bonus if university, observatory or laboratory in the city, 0.33 each)
        iRandTech = gc.getASyncRand().get(len(lTechDiff), 'SpyMod')    
        iMaxPoints = unit.plot().getPlotCity().getPopulation() * 70

        iBonusPoints = 1.0
        if (unit.plot().getPlotCity().hasBuilding(24)): #"BUILDING_UNIVERSITY"
            iBonusPoints = iBonusPoints + 0.33
        if (unit.plot().getPlotCity().hasBuilding(25)): #"BUILDING_OBSERVATORY"
            iBonusPoints = iBonusPoints + 0.33
        if (unit.plot().getPlotCity().hasBuilding(26)): #"BUILDING_LABORATORY"
            iBonusPoints = iBonusPoints + 0.33
            
        iTotalTechPoints = 100 + gc.getASyncRand().get(iMaxPoints, 'SpyMod')
        iTotalTechPoints = int(iTotalTechPoints * iBonusPoints)
        
        iTechCost = myTeam.getResearchCost(lTechDiff[iRandTech])
        iCurrentTechPoints = myTeam.getResearchProgress(lTechDiff[iRandTech])

        # no bonus points if there are enough to steal an entire tech
        if (iTotalTechPoints > (iTechCost - iCurrentTechPoints)):
            iTotalTechPoints = iTechCost - iCurrentTechPoints

        # update the progress
        myTeam.changeResearchProgress(lTechDiff[iRandTech], iTotalTechPoints, unit.getOwner())

        # display
        if (iTotalTechPoints == iTechCost - iCurrentTechPoints):
            CyInterface().addMessage(unit.getOwner(),True,25,u"Your spy discovers the secret of %s!"
                %gc.getTechInfo(lTechDiff[iRandTech]).getDescription(),
                'AS2D_DISEASED',1,"Art/Interface/Buttons/Actions/StealTech.dds",ColorTypes(8),
                unit.plot().getPlotCity().getX(),
                unit.plot().getPlotCity().getY(),True,True)           
        else:
            CyInterface().addMessage(unit.getOwner(),True,25,u"Your spy learns %d points of %s!"
                %(iTotalTechPoints, gc.getTechInfo(lTechDiff[iRandTech]).getDescription()),
                'AS2D_DISEASED',1,"Art/Interface/Buttons/Actions/StealTech.dds",ColorTypes(8),
                unit.plot().getPlotCity().getX(),
                unit.plot().getPlotCity().getY(),True,True)

        # end of turn for the unit
        unit.finishMoves()


# the steal tech button
stealTechButton = ActionButtons.ActionButton("Art/Interface/Buttons/Actions/StealTech.dds",stealTech)

# condition to have this button
def buttons(unit):
    # only for Spy
    if(gc.getUnitInfo(unit.getUnitType()).getType()!="UNIT_SPY"):
        return []
    # only in city
    currentPlot = unit.plot()
    if (currentPlot.isCity()==False):
        return []
    # only in ennemy's land
    if (currentPlot.getTeam()==unit.getTeam()):
        return []
    # visible but not active if no more move or not enough money
    if (unit.canMove()==False or gc.getPlayer(unit.getOwner()).getGold() < 150 ):
        stealTechButton.setEnabled(False)
        return [stealTechButton]

    # not active if there's no technology to steal
    lTechDiff = []
    targetTeam = gc.getTeam(currentPlot.getTeam())
    myTeam = gc.getTeam(unit.getTeam())
    myPlayer = gc.getPlayer(unit.getOwner())
    a = 0
    for iLoopTech in range(gc.getNumTechInfos()):
        if ((targetTeam.isHasTech(iLoopTech)) and (not myTeam.isHasTech(iLoopTech))
            and (myPlayer.canResearch(iLoopTech, False))):
            lTechDiff.append(iLoopTech)
    stealTechButton.setEnabled(len(lTechDiff)>0)
    
    return [stealTechButton]

ActionButtons.setButtonsFunction(buttons)
